from typing import Text, Tuple

import pytest

from liketext import html, md


@pytest.mark.parametrize(
    "decoration,testfunction,args,excepted",
    [
        (html, "link", ("hello", "https://some.org"), '<a href="https://some.org">hello</a>'),
        (html, "bold", ("hello",), "<b>hello</b>"),
        (html, "italic", ("hello",), "<i>hello</i>"),
        (html, "code", ("hello",), "<code>hello</code>"),
        (html, "pre", ("hello",), "<pre>hello</pre>"),
        (
            html,
            "pre_language",
            ("hello", "world"),
            '<pre><code class="world">hello</code></pre>',
        ),
        (html, "underline", ("hello",), "<u>hello</u>"),
        (html, "strikethrough", ("hello",), "<s>hello</s>"),
        (html, "spoiler", ("hello",), "<tg-spoiler>hello</tg-spoiler>"),
        (
            html,
            "custom_emoji",
            ("hello", "world"),
            '<tg-emoji emoji-id="world">hello</tg-emoji>',
        ),
        (html, "quote", ("<hello>",), "&lt;hello&gt;"),
        (md, "link", ("hello", "https://some.org"), "[hello](https://some.org)"),
        (md, "bold", ("hello",), "*hello*"),
        (md, "italic", ("hello",), "_hello_"),
        (md, "code", ("hello",), "`hello`"),
        (md, "pre", ("hello",), "```\nhello\n```"),
        (md, "pre_language", ("hello", "world"), "```world\nhello\n```"),
        (md, "underline", ("hello",), "__hello__"),
        (md, "strikethrough", ("hello",), "~hello~"),
        (md, "spoiler", ("hello",), "|hello|"),
        (md, "custom_emoji", ("hello", "world"), "[hello](tg://emoji?id=world)"),
        (md, "quote", ("[hello]",), "\\[hello\\]"),
    ],
)
def test_text_decorations(
    decoration: Text, testfunction: str, args: Tuple[str], excepted: str
) -> None:
    assert getattr(decoration, testfunction)(*args) == excepted



from typing import Any, Callable, Optional, Tuple

import pytest

from liketext import (
    bold,
    code,
    hbold,
    hcode,
    hide_link,
    hitalic,
    hlink,
    hpre,
    hstrikethrough,
    hunderline,
    italic,
    link,
    pre,
    strikethrough,
    text,
    underline,
)


@pytest.mark.parametrize(
    "testfunction,args,sep,excepted",
    [
        (text, ("test", "test"), " ", "test test"),
        (text, ("test", "test"), None, "test test"),
        (bold, ("test", "test"), " ", "*test test*"),
        (hbold, ("test", "test"), " ", "<b>test test</b>"),
        (italic, ("test", "test"), " ", "_test test_"),
        (hitalic, ("test", "test"), " ", "<i>test test</i>"),
        (code, ("test", "test"), " ", "`test test`"),
        (hcode, ("test", "test"), " ", "<code>test test</code>"),
        (pre, ("test", "test"), " ", "```\ntest test\n```"),
        (hpre, ("test", "test"), " ", "<pre>test test</pre>"),
        (underline, ("test", "test"), " ", "__test test__"),
        (hunderline, ("test", "test"), " ", "<u>test test</u>"),
        (strikethrough, ("test", "test"), " ", "~test test~"),
        (hstrikethrough, ("test", "test"), " ", "<s>test test</s>"),
        (link, ("test", "https://some.org"), None, "[test](https://some.org)"),
        (hlink, ("test", "https://some.org"), None, '<a href="https://some.org">test</a>'),
        (hide_link, ("https://some.org",), None, '<a href="https://some.org">&#8203;</a>'),
    ],
)
def test_formatter(
    testfunction: Callable[..., Any], args: Tuple[str], sep: Optional[str], excepted: str
) -> None:
    kwargs = {} if sep is None else {"sep": sep}
    assert testfunction(*args, **kwargs) == excepted



from .text import (
    bold,
    code,
    hbold,
    hcode,
    hide_link,
    hitalic,
    hlink,
    hpre,
    hstrikethrough,
    hunderline,
    italic,
    link,
    pre,
    strikethrough,
    text,
    underline,
)
from .text_decorations import html, md

__version__ = "1.1"
__all__ = (
    "text",
    "bold",
    "hbold",
    "italic",
    "hitalic",
    "code",
    "hcode",
    "pre",
    "hpre",
    "underline",
    "hunderline",
    "strikethrough",
    "hstrikethrough",
    "link",
    "hlink",
    "hide_link",
    "md",
    "html",
)



import abc
import re
from html import escape
from typing import Dict

from .re import MD_QUOTE_PATTERN


class Text(abc.ABC):
    _decorations: Dict[str, str]

    def link(self, value: str, link: str) -> str:
        decoration = self._decorations["link"]
        return decoration.format(link=link, value=value)

    def bold(self, value: str) -> str:
        decoration = self._decorations["bold"]
        return decoration.format(value=value)

    def italic(self, value: str) -> str:
        decoration = self._decorations["italic"]
        return decoration.format(value=value)

    def code(self, value: str) -> str:
        decoration = self._decorations["code"]
        return decoration.format(value=value)

    def pre(self, value: str) -> str:
        decoration = self._decorations["pre"]
        return decoration.format(value=value)

    def pre_language(self, value: str, language: str) -> str:
        decoration = self._decorations["pre_language"]
        return decoration.format(language=language, value=value)

    def underline(self, value: str) -> str:
        decoration = self._decorations["underline"]
        return decoration.format(value=value)

    def strikethrough(self, value: str) -> str:
        decoration = self._decorations["strikethrough"]
        return decoration.format(value=value)

    def spoiler(self, value: str) -> str:
        decoration = self._decorations["spoiler"]
        return decoration.format(value=value)

    def custom_emoji(self, value: str, custom_emoji_id: str) -> str:
        decoration = self._decorations["emoji"]
        return decoration.format(custom_emoji_id=custom_emoji_id, value=value)

    @abc.abstractmethod
    def quote(self, value: str) -> str:
        ...


class HTML(Text):
    _decorations: Dict[str, str] = {
        "link": '<a href="{link}">{value}</a>',
        "bold": "<b>{value}</b>",
        "italic": "<i>{value}</i>",
        "code": "<code>{value}</code>",
        "pre": "<pre>{value}</pre>",
        "pre_language": '<pre><code class="{language}">{value}</code></pre>',
        "underline": "<u>{value}</u>",
        "strikethrough": "<s>{value}</s>",
        "spoiler": "<tg-spoiler>{value}</tg-spoiler>",
        "emoji": '<tg-emoji emoji-id="{custom_emoji_id}">{value}</tg-emoji>',
    }

    def quote(self, value: str) -> str:
        return escape(value, quote=False)


class MD(Text):
    _decorations: Dict[str, str] = {
        "link": "[{value}]({link})",
        "bold": "*{value}*",
        "italic": "_{value}_",
        "code": "`{value}`",
        "pre": "```\n{value}\n```",
        "pre_language": "```{language}\n{value}\n```",
        "underline": "__{value}__",
        "strikethrough": "~{value}~",
        "spoiler": "|{value}|",
        "emoji": "[{value}](tg://emoji?id={custom_emoji_id})",
    }

    def quote(self, value: str) -> str:
        return re.sub(pattern=MD_QUOTE_PATTERN, repl=r"\\\1", string=value)


html = HTML()
md = MD()



from typing import Any

from .text_decorations import html, md


def _join(*content: Any, sep: str = " ") -> str:
    return sep.join(map(str, content))


def text(*content: Any, sep: str = " ") -> str:
    """
    Join all elements with a separator.
    """

    return _join(*content, sep=sep)


def bold(*content: Any, sep: str = " ") -> str:
    """
    Make bold text (Markdown).
    """

    return md.bold(value=md.quote(_join(*content, sep=sep)))


def hbold(*content: Any, sep: str = " ") -> str:
    """
    Make bold text (HTML).
    """

    return html.bold(value=html.quote(_join(*content, sep=sep)))


def italic(*content: Any, sep: str = " ") -> str:
    """
    Make italic text (Markdown).
    """

    return md.italic(value=html.quote(_join(*content, sep=sep)))


def hitalic(*content: Any, sep: str = " ") -> str:
    """
    Make italic text (HTML).
    """

    return html.italic(value=html.quote(_join(*content, sep=sep)))


def code(*content: Any, sep: str = " ") -> str:
    """
    Make mono-width text (Markdown).
    """

    return md.code(value=md.quote(_join(*content, sep=sep)))


def hcode(*content: Any, sep: str = " ") -> str:
    """
    Make mono-width text (HTML).
    """

    return html.code(value=html.quote(_join(*content, sep=sep)))


def pre(*content: Any, sep: str = "\n") -> str:
    """
    Make mono-width text block (Markdown).
    """

    return md.pre(value=md.quote(_join(*content, sep=sep)))


def hpre(*content: Any, sep: str = "\n") -> str:
    """
    Make mono-width text block (HTML).
    """

    return html.pre(value=html.quote(_join(*content, sep=sep)))


def underline(*content: Any, sep: str = " ") -> str:
    """
    Make underlined text (Markdown).
    """

    return md.underline(value=md.quote(_join(*content, sep=sep)))


def hunderline(*content: Any, sep: str = " ") -> str:
    """
    Make underlined text (HTML).
    """

    return html.underline(value=html.quote(_join(*content, sep=sep)))


def strikethrough(*content: Any, sep: str = " ") -> str:
    """
    Make strikethrough text (Markdown).
    """

    return md.strikethrough(value=md.quote(_join(*content, sep=sep)))


def hstrikethrough(*content: Any, sep: str = " ") -> str:
    """
    Make strikethrough text (HTML).
    """

    return html.strikethrough(value=html.quote(_join(*content, sep=sep)))


def link(title: str, url: str) -> str:
    """
    Format URL (Markdown).
    """

    return md.link(value=md.quote(title), link=url)


def hlink(title: str, url: str) -> str:
    """
    Format URL (HTML).
    """

    return html.link(value=html.quote(title), link=url)


def hide_link(url: str) -> str:
    """
    Hide URL (HTML only).
    Can be used for adding an image to a text message.
    """

    return f'<a href="{url}">&#8203;</a>'
